import { ContextTracker, ExternalTokenizer, LRParser } from '@lezer/lr';
import { LRLanguage, indentNodeProp, delimitedIndent, foldNodeProp, foldInside, LanguageSupport } from '@codemirror/language';
import { styleTags, tags } from '@lezer/highlight';

// This file was generated by lezer-generator. You probably shouldn't edit it.
const QuotedTemplateStart = 1,
  QuotedTemplateEnd = 2,
  TemplateInterpolationStart = 3,
  TemplateInterpolationEnd = 4,
  TemplateDirectiveStart = 5,
  TemplateDirectiveEnd = 6,
  HeredocIdentifier = 7,
  templateLiteralChunk = 77;

const ContextType = {
  QUOTED_TEMPLATE: 1,
  TEMPLATE_INTERPOLATION: 2,
  TEMPLATE_DIRECTIVE: 4,
  HEREDOC_TEMPLATE: 8,
};

const hashStr = function (str) {
  var hash = 0,
    i,
    chr;
  if (str.length === 0) return hash;
  for (i = 0; i < str.length; i++) {
    chr = str.charCodeAt(i);
    hash = (hash << 5) - hash + chr;
    hash |= 0; // Convert to 32bit integer
  }
  return hash;
};

class Context {
  constructor(parent, type, heredoc_identifier) {
    this.parent = parent;
    this.type = type;
    this.heredoc_identifier = heredoc_identifier;
    this.hash =
      (parent ? (parent.hash + parent.hash) << 8 : 0) +
      (type << 4) +
      (this.heredoc_identifier === "" ? 0 : hashStr(this.heredoc_identifier));
  }
}

const topCtx = new Context(null, "", "");

const trackTemplate = new ContextTracker({
  start: topCtx,
  shift(context, term, stack, input) {
    if (term === QuotedTemplateStart) {
      return new Context(context, ContextType.QUOTED_TEMPLATE, "");
    }
    if (term === TemplateInterpolationStart) {
      return new Context(context, ContextType.TEMPLATE_INTERPOLATION, "");
    }
    if (term === TemplateDirectiveStart) {
      return new Context(context, ContextType.TEMPLATE_DIRECTIVE, "");
    }
    if (
      term === HeredocIdentifier &&
      context.type !== ContextType.HEREDOC_TEMPLATE
    ) {
      const ctx = new Context(
        context,
        ContextType.HEREDOC_TEMPLATE,
        input.read(input.pos, stack.pos)
      );
      return ctx;
    }

    if (
      term === HeredocIdentifier &&
      context.type === ContextType.HEREDOC_TEMPLATE
    ) {
      return context.parent;
    }
    switch (term) {
      case QuotedTemplateEnd:
      case TemplateInterpolationEnd:
      case TemplateDirectiveEnd:
        return context.parent;
    }

    return context;
  },
  hash(context) {
    return context.hash;
  },
});

// export const insertSemicolon = new ExternalTokenizer(
//   (input, stack) => {
//     let { next } = input;
//     if (next == braceR || next == -1 || stack.context)
//       input.acceptToken(insertSemi);
//   },
//   { contextual: true, fallback: true }
// );

const scanQuotedTemplateStart = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(QuotedTemplateStart) &&
      !(stack.context && stack.context.type === ContextType.QUOTED_TEMPLATE) &&
      String.fromCharCode(input.next) === '"'
    ) {
      input.advance();
      return input.acceptToken(QuotedTemplateStart);
    }
  },
  { contextual: true, fallback: true }
);
const scanQuotedTemplateEnd = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(QuotedTemplateEnd) &&
      stack.context &&
      stack.context.type === ContextType.QUOTED_TEMPLATE &&
      String.fromCharCode(input.next) === '"'
    ) {
      input.advance();
      return input.acceptToken(QuotedTemplateEnd);
    }
  },
  { contextual: true, fallback: true }
);

const scanTemplateLiteralChunk = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(templateLiteralChunk) &&
      stack.context &&
      stack.context.type === ContextType.QUOTED_TEMPLATE
    ) {
      switch (String.fromCharCode(input.next)) {
        case "\\":
          input.advance();
          switch (String.fromCharCode(input.next)) {
            case '"':
            case "n":
            case "r":
            case "t":
            case "\\":
              input.advance();
              return input.acceptToken(templateLiteralChunk);
            case "u":
              for (let i = 0; i < 4; i++) {
                input.advance();
                if (!/[0-9a-fA-F]/.test(String.fromCharCode(input.next))) {
                  return;
                }
              }
              input.advance();
              return input.acceptToken(templateLiteralChunk);
            case "U":
              for (let i = 0; i < 8; i++) {
                input.advance();
                if (!/[0-9a-fA-F]/.test(String.fromCharCode(input.next))) {
                  return;
                }
              }
              input.advance();
              return input.acceptToken(templateLiteralChunk);
            default:
              return;
          }
      }
    }

    if (
      stack.canShift(templateLiteralChunk) &&
      stack.context &&
      (stack.context.type === ContextType.QUOTED_TEMPLATE ||
        stack.context.type === ContextType.HEREDOC_TEMPLATE) &&
      input.next !== -1
    ) {
      input.advance();
      return input.acceptToken(templateLiteralChunk);
    }
  },
  { contextual: true, fallback: true }
);
const scanTemplateInterpolationStart = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(TemplateInterpolationStart) &&
      stack.canShift(templateLiteralChunk) &&
      !(
        stack.context &&
        stack.context.type === ContextType.TEMPLATE_INTERPOLATION
      ) &&
      String.fromCharCode(input.next) === "$"
    ) {
      input.advance();
      if (String.fromCharCode(input.next) === "{") {
        input.advance();
        return input.acceptToken(TemplateInterpolationStart);
      }

      // escape sequence
      if (String.fromCharCode(input.next) === "$") {
        input.advance();
        if (String.fromCharCode(input.next) === "{") {
          // $${
          input.advance();
          return input.acceptToken(templateLiteralChunk);
        }
      }

      return input.acceptToken(templateLiteralChunk);
    }
  },
  { contextual: true, fallback: true }
);
const scanTemplateInterpolationEnd = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(TemplateInterpolationEnd) &&
      stack.context &&
      stack.context.type === ContextType.TEMPLATE_INTERPOLATION &&
      String.fromCharCode(input.next) === "}"
    ) {
      input.advance();
      return input.acceptToken(TemplateInterpolationEnd);
    }
  },
  { contextual: true, fallback: true }
);
const scanTemplateDirectiveStart = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(TemplateDirectiveStart) &&
      stack.canShift(templateLiteralChunk) &&
      !(
        stack.context && stack.context.type === ContextType.TEMPLATE_DIRECTIVE
      ) &&
      String.fromCharCode(input.next) === "%"
    ) {
      input.advance();
      if (String.fromCharCode(input.next) === "{") {
        input.advance();
        return input.acceptToken(TemplateDirectiveStart);
      }

      // escape sequence
      if (String.fromCharCode(input.next) === "%") {
        input.advance();
        if (String.fromCharCode(input.next) === "{") {
          // $${
          input.advance();
          return input.acceptToken(templateLiteralChunk);
        }
      }

      return input.acceptToken(templateLiteralChunk);
    }
  },
  { contextual: true, fallback: true }
);
const scanTemplateDirectiveEnd = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(TemplateDirectiveEnd) &&
      stack.context &&
      stack.context.type === ContextType.TEMPLATE_DIRECTIVE &&
      String.fromCharCode(input.next) === "}"
    ) {
      input.advance();
      return input.acceptToken(TemplateDirectiveEnd);
    }
  },
  { contextual: true, fallback: true }
);

const scanHeredocIdentifier = new ExternalTokenizer(
  (input, stack) => {
    if (
      stack.canShift(HeredocIdentifier) &&
      !(stack.context && stack.context.type === ContextType.HEREDOC_TEMPLATE) &&
      input.next !== -1
    ) {
      let next = String.fromCharCode(input.next);
      while (/[0-9a-zA-Z]/.test(next) || next === "_" || next === "-") {
        input.advance();
        next = String.fromCharCode(input.next);
      }

      return input.acceptToken(HeredocIdentifier);
    }

    if (
      stack.canShift(HeredocIdentifier) &&
      stack.canShift(templateLiteralChunk) &&
      stack.context &&
      stack.context.type === ContextType.HEREDOC_TEMPLATE &&
      input.next != -1
    ) {
      const expected = stack.context.heredoc_identifier;

      let has_leading_whitespace_with_newline = false;
      let has_leading_whitespace = false;
      let next = String.fromCharCode(input.next);
      while (/\s/.test(next)) {
        has_leading_whitespace = true;
        if (next === "\n") {
          has_leading_whitespace_with_newline = true;
        }
        input.advance();
        next = String.fromCharCode(input.next);
      }
      if (has_leading_whitespace) {
        input.acceptToken(templateLiteralChunk);
      }

      if (!has_leading_whitespace_with_newline) {
        return;
      }

      let advance = 0;
      for (let i = 0; i < expected.length; i++) {
        if (String.fromCharCode(input.next) === expected[i]) {
          input.advance();
          advance++;
        } else {
          if (advance) {
            input.acceptToken(templateLiteralChunk);
          }
          return
        }
      }

      // check if the identifier is on a line of its own
      let i = 0;
      while (true) {
        const ch = input.peek(i);
        if (ch == -1 || String.fromCharCode(ch) === "\n") {
          return input.acceptToken(HeredocIdentifier);
        }

        if (!/\s/.test(String.fromCharCode(ch))) {
          return input.acceptToken(templateLiteralChunk);
        }

        i++;
      }
    }
  },
  { contextual: true, fallback: true }
);

// This file was generated by lezer-generator. You probably shouldn't edit it.
const spec_Identifier = {__proto__:null,true:39, false:41, null:45, for:64, in:66, endfor:70, if:76, else:80, endif:84};
const parser = LRParser.deserialize({
  version: 14,
  states: "DlQ]Q&jOOPeO&jOOOmQ&kO'#D]O!kQ&kO'#CiOOQO'#Dt'#DtO!yQ&jO'#ChQOQ&jOOPOOO'#Cd'#CdPOOO)C?r)C?rOOQy'#Cn'#CnOOQy'#Cq'#CqO#UQ)vO'#CvOOQy'#Cl'#ClOOQ!b'#DY'#DYO#dQ'|O'#DXOOQy'#Cu'#CuO$gQ&kO'#D[O$nQ&jO'#D^O%tQ&kO'#EZO&OQ&kO'#D]OOQy,59w,59wO&VQ&jO,59wOOQy'#DZ'#DZO'lQ'dO'#D_OOQy'#Dc'#DcO$yQ&kO'#DiO)aQ'dO'#CkOOQy'#Dh'#DhOOQy'#EP'#EPO$yQ&kO'#EPOOQy'#Ck'#CkO#_Q)`O'#CsO$yQ&kO,59TOOQP'#Dy'#DyO*sQ&kO,5:_O+OQ&jO,5:_OOQO-E7r-E7rO+WQ+YO'#CtO+iQ)vO,59_O+wQ&sO'#CwO,RQ)tO'#CzO,^Q)tO'#DQOOQ%n'#Cy'#CyOOQ%n'#Du'#DuO,iQ)vO,59bOOQy,59b,59bO,wQ&jO'#DRO-SQ+WO,59sO-bQ&jO'#EWOOQy,59v,59vO-mQ&jO,59vO$yQ&kO,5:OO-rQ&jO'#DeO-wQ&kO,59xO$yQ&kO,5:^OOQP'#Dw'#DwO.rQ&kO,5:uO.yQ&kO,5:uO/TQ&kO,5:ROOQy1G/c1G/cO0OQ&kO,59zOOQy,5:T,5:TO$yQ&kO,5:UO$yQ&kO,5:UO$yQ&kO,5:UO$yQ&kO,5:UO$yQ&kO,5:UO$yQ&kO,5:UO$yQ&kO'#DlOOQy'#Dk'#DkO0VQ&jO'#DnOOQy'#Dp'#DpOOQy'#Dq'#DqOOQy'#Do'#DoOOQy,5:k,5:kO0[Q&jO,5:kO0dQ&lO,59_O0iQ&jO1G.oOOQP-E7w-E7wO0wQ&jO1G/yOOQO1G/y1G/yO0zQ&jO1G/yO+WQ+YO'#EQOOQ%n,59`,59`OOQy1G.y1G.yOOQ%n,59c,59cO1PQ&sO,59cO1ZQ&rO,59cOOQ%n,59f,59fO,RQ)tO,59fO1fQ&jO'#DOOOQ%n,59l,59lO1tQ)tO,59lO,^Q)tO,59lO2PQ&jO'#DTOOQ%n-E7s-E7sOOQy1G.|1G.|O2bQ&jO,59gO2gQ&jO,59mO$yQ&kO,59mO2oQ+WO1G/_OOQy1G/_1G/_O2}Q&kO,5:rO3UQ&jO,5:rOOQy1G/b1G/bO3^Q&jO1G/jO3iQ&jO,5:PO3qQ&kO1G/dOOQP'#D_'#D_O4uQ&jO1G/xOOQP,5:c,5:cO4}Q&kO1G0aOOQP-E7u-E7uO5UQ,UO'#D_O5]Q,UO1G/mO6]Q,UO'#CkO6yQ&jO'#DaOOQy1G/f1G/fO7XQ&jO1G/fOOQy1G/p1G/pO9^Q'dO1G/pO9hQ'dO1G/pO;rQ'dO1G/pO;|Q'dO1G/pO<WQ'dO1G/pO=jQ&jO,5:WOOQy,5:Y,5:YPOQy'#Dx'#DxPOQy,5:[,5:[POQy,5:],5:]OOQy1G0V1G0VOOQO7+%e7+%eO=rQ&jO7+%eOOQ%n,5:l,5:lOOQ%n1G.}1G.}O=wQ&rO1G.}O=|Q&rO1G.}OOQ%n1G/Q1G/QO>XQ&jO,59jO>dQ'[O,59jOOQ%n1G/W1G/WO1tQ)tO1G/WO>lQ&jO'#DVO1tQ)tO1G/WO>zQ&jO,59oO?YQ'[O,59oO?bQ'[O,59qO?jQ&jO1G/RO?rQ&jO1G/RO$yQ&kO1G/XO?wQ'[O1G/XOOQy7+$y7+$yO@SQ&jO,5:bO@eQ&kO1G0^OOQO-E7t-E7tOOQy7+%U7+%UO@lQ&jO7+%UO$yQ&kO'#DfO$yQ&kO1G/kO@qQ&jO1G/kO$yQ&kO7+%dP$yQ&kO'#DwO$yQ&kO7+%XO@vQ&kO,59{O@}Q&jO,59{OOQO,59{,59{OOQy7+%Q7+%QOOQy1G/r1G/rPOQy-E7v-E7vOOQO<<IP<<IPOOQ%n7+$i7+$iOAYQ&rO7+$iOA_Q'[O1G/UOOQ%n1G/U1G/UOAgQ'[O1G/UOOQ%n7+$r7+$rOAlQ&jO,59qO1tQ)tO7+$rOAwQ'[O1G/ZOBPQ'[O1G/]OOQ$Y1G/Z1G/ZOBXQ'[O1G/ZOOQ%n1G/]1G/]OB^Q'[O1G/]O$yQ&kO7+$mOBcQ&jO7+$mOBhQ&jO7+$mOBpQ'[O7+$sOOQ$Y7+$s7+$sOB{Q'[O7+$sP$yQ&kO'#DvOOQy<<Hp<<HpOCQQ&jO,5:QOC]Q&jO7+%VOCeQ&jO7+%VOOQy<<IO<<IOOCjQ&jO<<HsOESQ,UO1G/pOE^Q,UO1G/pOFrQ,UO1G/pOF|Q,UO1G/pOGWQ,UO1G/pOGtQ&kO1G/gOOQO1G/g1G/gOOQ%n<<HT<<HTOOQ%n7+$p7+$pOG{Q'[O7+$pOOQ%n<<H^<<H^OOQ$Y7+$u7+$uOHQQ'[O7+$uOOQ%n7+$w7+$wOHVQ'[O7+$wOH[Q'[O<<HXOHgQ&jO<<HXO$yQ&kO<<HXOHlQ&jO<<HXOOQ$Y<<H_<<H_OHqQ'[O<<H_OOQP<<Hq<<HqO$yQ&kO<<HqOOQyAN>_AN>_OHvQ&jOAN>_OH{Q&jOAN>_OOQ%n<<H[<<H[OOQ$Y<<Ha<<HaOOQ%n<<Hc<<HcOOQ$YAN=sAN=sOITQ'[OAN=sO$yQ&kOAN=sOIYQ'[OAN=sOIeQ&jOAN=sOOQ$YAN=yAN=yOIjQ&jOAN>]OOQyG23yG23yOIrQ&jOG23yOOQ$YG23_G23_OIwQ'[OG23_OJSQ'[OG23_O$yQ&kOG23_OOQPG23wG23wOOQyLD)eLD)eOOQ$YLD(yLD(yOJXQ'[OLD(yOJ^Q'[OLD(yOOQ$Y!$'Le!$'LeOJiQ'[O!$'LeOOQ$Y!)9BP!)9BPO-wQ&kO'#DiO/TQ&kO'#DiOKfQ&kO'#CkO-wQ&kO,5:UO/TQ&kO,5:UO-wQ&kO,5:UO/TQ&kO,5:UO-wQ&kO,5:UO/TQ&kO,5:UO-wQ&kO,5:UO/TQ&kO,5:UO-wQ&kO,5:UO/TQ&kO,5:UO-wQ&kO,5:UO/TQ&kO,5:UONxQ&kO1G/pO! SQ&kO1G/pO!#^Q&kO1G/pO!#hQ&kO1G/pO!#rQ&kO1G/pO-wQ&kO7+%dO/TQ&kO7+%dO!%UQ&jO1G/xO!%^Q&jO1G/xO$yQ&kO,5:^O$yQ&kO,5:^",
  stateData: "!%n~O!qOSXPQYPQ~O^RO!|QO~OXVOYVO~OPZO^gOa[OcXOdXOfYO!v]O!w]O!y`O!{dO!|cO#QmO#SiO#TiO~OPoO^qO!rpO!|sO~O^RO!n[X!{[X~OQ}ORwOT!OO!ouO~OV!PO~OPZO^gOa[OcXOdXOfYOp!UO!v]O!w]O!y`O!|cO#QmO#SiO#TiO~O!x!RO~P#iO!r!VO#O!VO#e!WO~OPZO^gOa[OcXOdXOfYO!v]O!w]O!y`O!|cO#QmO#SiO#TiO~O!u!YO!{!}X~P$yOp!UO~PmO!{!]O~O#Q!^O!a!RX!y!RX#S!RX#U!RX#V!RX#W!RX#X!RX#Y!RX#Z!RX#[!RX#]!RX#^!RX#_!RX#`!RX#a!RX#b!RX#c!RX#d!RX#e!RX~O!r!RX#O!RX!u!RX!x!RX#P!RX^!RX!n!RXS!RXl!RXv!RX!U!RXU!RX!{!RX~P&[O#S!aO#U!`O#V!`O#W!`O#X!aO#Y!bO#Z!bO#[!bO#]!bO#^!cO#_!cO#`!dO~O!a!gO!y!fO#a!eO#b!hO#c!iO#d!jO!r_X#O_X#e_X!u_X!x_X#P_X^_X!n_XS_Xl_Xv_X!U_XU_X!{_X~P(iOPoO^qO!|!qO~O^RO!{!rO~O!o!tOQ!tPR!tPT!tPV!tP~OQ!vOR!iXT!iX!o!iX~OS!wOl!xO~P$yORwOT!|O!ouO~ORwOT#QO!ouO~OQ#SORwOT!OO!ouO~Ol#UOp#TOv#VO~ORwOT!OOV#XO!ouO~O!u#YO#e!WO!x!zX~O!x#[O~O^#^O~OPZO^#`Oa[OcXOdXOfYO!v]O!w]O!y`O!|cO#QmO#S&aO#T&aO~O!{!}a~P$yO!u#cO!{!}a~P$yOPZO^#eOa[OcXOdXOfYO!v]O!w]O!y`O!|cO#QmO#S&bO#T&bO~O#P#iO~P$yO^#rO~O#P#vO#e!WO~OQ!vO~O#e!WO^]i!n]i!{]i~O^RO!{#wO~OS#zOl#{O~P$yOS#zOl#{O#e!WO~Ol$OOp#TOs$POv#VO~ORwOT$SO!ouO~Ol$UOp#TOv#VOx$VOz$WO~O^$XO~Op$YOv$ZO~ORwOT!OOV$]O!ouO~O!x!za~P$yO!u$_O!x!za~Ov$cO!x$aO#e!WO~Oq$dO!u$eO~O#e&yOP!Qi^!Qia!Qic!Qid!Qif!Qi!u!Qi!v!Qi!w!Qi!y!Qi!{!Qi!|!Qi#Q!Qi#S!Qi#T!Qi~O#O$fO#e!WO~O!{!}i~P$yO#R!RX~P&[O#R$hO#e&zO~O#S&gO#U&eO#V&eO#W&eO#X&gO#Y&iO#Z&iO#[&iO#]&iO#^&kO#_&kO#`&mO~O!a!gO!y!fO#a&oO#b!hO#c!iO#d!jO#R_X#e_X~P5eO!U$kO!u$iO#e!WO#P!TX~O#P$lO~O#U!`O#V!`O#W!`O!a!^i!r!^i!y!^i#O!^i#Y!^i#Z!^i#[!^i#]!^i#^!^i#_!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i!u!^i!x!^i#P!^i^!^i!n!^iS!^il!^iv!^i!U!^iU!^i!{!^i~O#S!^i#X!^i~P7^O#S!aO#X!aO~P7^O#S!aO#U!`O#V!`O#W!`O#X!aO#Y!bO#Z!bO#[!bO#]!bO!a!^i!r!^i!y!^i#O!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i!u!^i!x!^i#P!^i^!^i!n!^iS!^il!^iv!^i!U!^iU!^i!{!^i~O#^!^i#_!^i~P9rO#^!cO#_!cO~P9rO!a!^i!r!^i!y!^i#O!^i#a!^i#b!^i#c!^i#d!^i#e!^i!u!^i!x!^i#P!^i^!^i!n!^iS!^il!^iv!^i!U!^iU!^i!{!^i~P(iO!x$mO#e!WO~O!{$oO~OS$pO~OS$pOl$qO#e!WO~Op$YOs$rOv$ZO~OU$sOl$tO~Ol$vOp#TOv#VOz$WO~Op$YOv$ZOx$xOz$yO~OU$zOl${O~OU$|Ol$}O~Oq%OO!u%PO~O^%QO~OU%SOl%TO#e!WO~O#e!WO!u!ja!x!ja!U!ja#P!ja~O!x!zi~P$yO!x%VO~O^%YO~O#P!Ta~P$yO!U%cO!u%bO#P!Ta~OS%dO~OU%eOl%fO~OU%eO~Op$YOv$ZOz$yO~OU%hOl%iO~OU%jOl%kO~OU%hO~OU%jO~O^%mO~Oq%nO!u%oO~OU%pOl%qO#e!WO~OU%pO~O#e!WO!x!Ya!{!Ya~O#O%rO#e!WO~Oq%sO~Ov$cO!U%vO!{%tO#e!WO~O#U&eO#V&eO#W&eO!a!^i!y!^i#R!^i#Y!^i#Z!^i#[!^i#]!^i#^!^i#_!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i~O#S!^i#X!^i~PCxO#S&gO#X&gO~PCxO#S&gO#U&eO#V&eO#W&eO#X&gO#Y&iO#Z&iO#[&iO#]&iO!a!^i!y!^i#R!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i~O#^!^i#_!^i~PEhO#^&kO#_&kO~PEhO!a!^i!y!^i#R!^i#a!^i#b!^i#c!^i#d!^i#e!^i~P5eO#P!Ti~P$yOU%wO~OU%xO~OU%yO~OU%zOl%{O#e!WO~Oq%|O~O^&OO~OU&PO~O!{&RO~Ov$cO!{&RO~OU&TO~OU&TOl&VO#e!WO~Oq&WO~O#O&XO#e!WO~O!{&YO~OU&ZOl&[O#e!WO~OU&ZO~OU&^O~OU&^Ol&_O#e!WO~OU&`O~O#S&fO#U&dO#V&dO#W&dO#X&fO#Y&hO#Z&hO#[&hO#]&hO#^&jO#_&jO#`&lO~O!a!gO#a&nO#b!hO#c!iO#d!jOP_X^_Xa_Xc_Xd_Xf_X!u_X!v_X!w_X!y_X!{_X!|_X#Q_X#T_X#e_X~PJnO#U&dO#V&dO#W&dOP!^i^!^ia!^ic!^id!^if!^i!a!^i!u!^i!v!^i!w!^i!y!^i!{!^i!|!^i#Q!^i#T!^i#Y!^i#Z!^i#[!^i#]!^i#^!^i#_!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i~O#S!^i#X!^i~PLxO#S&fO#X&fO~PLxO#S&fO#U&dO#V&dO#W&dO#X&fO#Y&hO#Z&hO#[&hO#]&hOP!^i^!^ia!^ic!^id!^if!^i!a!^i!u!^i!v!^i!w!^i!y!^i!{!^i!|!^i#Q!^i#T!^i#`!^i#a!^i#b!^i#c!^i#d!^i#e!^i~O#^!^i#_!^i~P! ^O#^&jO#_&jO~P! ^OP!^i^!^ia!^ic!^id!^if!^i!a!^i!u!^i!v!^i!w!^i!y!^i!{!^i!|!^i#Q!^i#T!^i#a!^i#b!^i#c!^i#d!^i#e!^i~PJnO#O&uO#e!WO~O#O&vO#e!WO~Oa!a#b^X#V^~",
  goto: "9V#OPPPPPPPP#PPPP#S#]P#c%aP&pPP&pP(P)d%a)w+WP+W+f+tPP,SP+f,YP,hP,nP)w,|%a.]/l1O%a%a1]P%a1`2o2u1`%a3O3O4_4dP4_4_4h4h4m#]5g5o6Z6eP6kPPPPP6q8xPPPPP9OPP9RRWPQUOQ!ssR#x!qXSOTs!q^aQbc!Y!Z#c$gQ!Q`Q!mmQ!opQ!ywQ#]!TQ#_!VQ#a!WQ#f![Q#h!^Q#q!fQ#|!xQ$[#VY$^#Y$_$i%U%bQ%R$ZQ%W$cQ%X$dU%Z$f&u&vQ%[$hQ%l%OQ%}%nQ&Q%sQ&U%|Q&]&WQ&w&yR&x&z#klQ`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&z#k[Q`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&z#j[Q`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zTqRrQvZf{xy|!P!{#O#P#W$R$T$wR!no#k_Q`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zi{Zxy|!P!{#O#P#W$R$T$wizZxy|!P!{#O#P#W$R$T$wixZxy|!P!{#O#P#W$R$T$wQ!zxR#}!{iyZxy|!P!{#O#P#W$R$T$wQ#OyR$T#PQ!}yS$Q#O#PS$u$R$TR%g$w#k^Q`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&z#kfQ`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zQUO#kfQ`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zSbQcS!Xb!ZV#b!Y#c$gR#j!^#khQ`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zQ!T`R![cQ$b#]Q%u%[R&S%v#kkQ`bcimpw!T!V!W!Y!Z![!^!`!a!b!c!d!e!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&a&b&d&e&f&g&h&i&j&k&l&m&n&o&u&v&y&zV!lj#g&cT!gj#gV!kj#g&c!onQ`bcmpw!T!V!W!Y!Z![!^!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&u&v&y&zUTOs!qRtTQ|ZQ!{xQ#Py[#R|!{#P#W$R$wQ#W!PQ$R#OR$w$TQ#Z!QS$`#Z$jR$j#hQ!ZbR#d!ZQrRR!pr!fjQ`bcmpw!T!W!Y!Z!^!f!x#V#Y#c$Z$_$c$d$f$g$h$i%O%U%b%n%s%|&W&y&zU!_i&a&bS#g![&vU#k!`&d&eQ#l!aQ#m!bQ#n!cQ#o!dQ#p!eQ%]&gQ%^&iQ%_&kQ%`&mQ%a&oS&c!V&uQ&p&fQ&q&hQ&r&jQ&s&lR&t&nQ!uuR#y!tR!S`TeQc",
  nodeNames: "⚠ QuotedTemplateStart QuotedTemplateEnd TemplateInterpolationStart TemplateInterpolationEnd TemplateDirectiveStart TemplateDirectiveEnd HeredocIdentifier Comment LineComment BlockComment ConfigFile Body Attribute Identifier Expression LiteralValue NumericLit BoolLit true false NullLit null StringLit TemplateLiteral TemplateExpr QuotedTemplate TemplateInterpolation StripMarker TemplateDirective TemplateFor TemplateForStart for in TemplateForEnd endfor TemplateIf TemplateIfIntro if TemplateElseIntro else TemplateIfEnd endif HeredocTemplate HeredocStart CollectionValue Tuple Object ObjectElem VariableExpr FunctionCall FunctionArguments Ellipsis ForExpr ForTupleExpr ForIntro ForCond ForObjectExpr Operation UnaryOperation BinaryOperation Index NewIndex LegacyIndex GetAttr Splat AttrSplat FullSplat Conditional Block",
  maxTerm: 113,
  context: trackTemplate,
  skippedNodes: [0,8,9,10],
  repeatNodeCount: 6,
  tokenData: "-h~RvX^#ipq#iqr$^st$kuv%Svw%Xxy%dyz%iz{%n{|%s|}%x}!O%}!O!P&S!P!Q&z!Q!R(^!R![)l![!]*c!^!_*h!_!`+V!`!a+l!a!b+y!c!},O!}#O,d#P#Q,w#R#S,O#T#o,O#o#p,|#p#q-R#q#r-^#r#s-c#y#z#i$f$g#i#BY#BZ#i$IS$I_#i$I|$JO#i$JT$JU#i$KV$KW#i&FU&FV#i~#nY!q~X^#ipq#i#y#z#i$f$g#i#BY#BZ#i$IS$I_#i$I|$JO#i$JT$JU#i$KV$KW#i&FU&FV#i~$cP#T~!_!`$f~$kO#_~~$pSX~OY$kZ;'S$k;'S;=`$|<%lO$k~%PP;=`<%l$k~%XO#W~~%[Pvw%_~%dO#`~~%iO#Q~~%nO#P~~%sO#U~~%xO#X~~%}O!u~~&SO#S~~&XR#b~z{&b!O!P&g!Q![&r~&gO#c~~&jP!O!P&m~&rO!U~~&wP!a~!Q![&r~'PQ#V~z{'V!P!Q$k~'YTOz'Vz{'i{;'S'V;'S;=`(W<%lO'V~'lVOz'Vz{'i{!P'V!P!Q(R!Q;'S'V;'S;=`(W<%lO'V~(WOY~~(ZP;=`<%l'V~(cRa~!O!P(l!Q![)l#l#m)w~(oP!Q![(r~(wRa~!Q![(r!g!h)Q#X#Y)Q~)TR{|)^}!O)^!Q![)d~)aP!Q![)d~)iPa~!Q![)d~)qQa~!O!P(l!Q![)l~)zR!Q![*T!c!i*T#T#Z*T~*YRa~!Q![*T!c!i*T#T#Z*T~*hO#O~~*mQ#[~!^!_*s!_!`+Q~*xP!v~}!O*{~+QO!w~~+VO#]~~+[Q!rP!_!`+b!`!a+g~+gO#^~Q+lO#RQ~+qP#Y~!_!`+t~+yO#Z~~,OO#e~~,TT^~}!O,O!Q![,O!c!},O#R#S,O#T#o,O~,iP!y~z{,l~,oP#P#Q,r~,wO#d~~,|O!x~~-RO!|~~-UP#p#q-X~-^O#a~~-cO!{~~-hOl~",
  tokenizers: [scanQuotedTemplateStart, scanQuotedTemplateEnd, scanTemplateInterpolationStart, scanTemplateInterpolationEnd, scanTemplateDirectiveStart, scanTemplateDirectiveEnd, scanHeredocIdentifier, scanTemplateLiteralChunk, 0, 1],
  topRules: {"ConfigFile":[0,11]},
  dynamicPrecedences: {"18":1,"21":1},
  specialized: [{term: 14, get: (value) => spec_Identifier[value] || -1}],
  tokenPrec: 2322
});

const hclLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Application: delimitedIndent({ closing: ")", align: false })
            }),
            foldNodeProp.add({
                Application: foldInside
            }),
            styleTags({
                "for in if else endfor endif": tags.controlKeyword,
                "Attribute/Identifier": tags.attributeName,
                "Attribute/Expression": tags.attributeValue,
                "Block/Identifier": tags.typeName,
                "VariableExpr/Identifier": tags.variableName,
                Identifier: tags.name,
                BoolLit: tags.bool,
                "StringLit/...": tags.string,
                NumericLit: tags.number,
                NullLit: tags.null,
                Comment: tags.comment,
                LineComment: tags.lineComment,
                BlockComment: tags.blockComment,
                "( )": tags.paren,
                "{ }": tags.bracket,
                "[ ]": tags.squareBracket,
                TemplateInterpolationStart: tags.paren,
                TemplateInterpolationEnd: tags.paren,
                Ellipsis: tags.punctuation,
            }),
        ]
    }),
    languageData: {
        commentTokens: { line: ";" }
    }
});
function hcl() {
    return new LanguageSupport(hclLanguage);
}

export { hcl, hclLanguage };
